@@ -96,6 +96,15 @@ def execute(argv=None, settings=None):
         settings = get_project_settings()
     check_deprecated_settings(settings)
 
+    # --- backwards compatibility for scrapy.conf.settings singleton ---
+    import warnings
+    from scrapy.exceptions import ScrapyDeprecationWarning
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", ScrapyDeprecationWarning)
+        from scrapy import conf
+        conf.settings = settings
+    # ------------------------------------------------------------------
+
     crawler = CrawlerProcess(settings)
     crawler.install()
     inproject = inside_project()
